from ..core.channel_description import ChannelDescription
from ..core.mesh_description import MeshDescription
from ..core.aabb import BoundingBox


##---------------------------------------------------------------------------------------------------
class MeshBuilder:
    ##-----------------------------------------------------------------------------------------------
    def __init__(
        self, name, 
        uuid, submesh_count
        ):
        self.description_obj = MeshDescription(name, uuid, submesh_count
                                                                )
        
    ##-----------------------------------------------------------------------------------------------
    def register_channel(self, submesh_index, semantic, element_count, attribute_count, element_size, element_type):
        alignment         = element_size * element_count
        alignment_padding = self.description_obj.payload_size % alignment
        data_offset       = self.description_obj.payload_size + alignment_padding
        data_size         = attribute_count * element_count * element_size
      
        channel_description_obj = ChannelDescription(data_offset = data_offset, data_size = data_size, 
                                                              element_count = element_count, 
                                                              element_size = element_size,
                                                              element_type = element_type, 
                                                              semantic = semantic,
                                                              )

        self.description_obj.payload_size += data_size + alignment_padding
        
        if semantic == "index":
            assert attribute_count % 3 == 0
            self.description_obj.submeshes[submesh_index].index_count = attribute_count
        elif semantic == "position":
            assert element_count == 3
            self.description_obj.submeshes[submesh_index].vertex_count = attribute_count

        self.description_obj.submeshes[submesh_index].channels.append(
                                                        channel_description_obj)
        
        return len(self.description_obj.submeshes[submesh_index].channels) - 1
        
    ##-----------------------------------------------------------------------------------------------        
    def calculate_bounding_box(self, vertices, submesh_index):
        min_x = min([v[0] for v in vertices])
        max_x = max([v[0] for v in vertices])
        min_y = min([v[1] for v in vertices])
        max_y = max([v[1] for v in vertices])
        min_z = min([v[2] for v in vertices])
        max_z = max([v[2] for v in vertices])

        bb_min = [min_x, min_y, min_z]
        bb_max = [max_x, max_y, max_z]

        self.description_obj.submeshes[submesh_index].aabb = BoundingBox(bb_min, bb_max)

    ##----------------------------------------------------------------------------------------------- 
    def calculate_uv_distribution_metric(self, submesh_index, value):
        self.description_obj.submeshes[submesh_index].uv_distribution_metric = value
              