import bmesh


##---------------------------------------------------------------------------------------------------
class Submesh:
    ##-----------------------------------------------------------------------------------------------
    def __init__(self, blender_mesh):
        self.blender_mesh = blender_mesh.copy()
        self.triangulate_mesh()
        
        self.indices        = [[v for v in poly.vertices] for poly in self.blender_mesh.polygons]
        self.normals        = [[vertex.normal.x, vertex.normal.y, vertex.normal.z] for vertex in self.blender_mesh.vertices]
        self.positions      = [[vertex.co.x, vertex.co.y, vertex.co.z] for vertex in self.blender_mesh.vertices]
        
        self.num_indices    = len(self.indices) #polygon index
        self.num_positions  = len(self.positions) #vertex index
        
    ##-----------------------------------------------------------------------------------------------
    def has_normals(self):
        if self.normals:
            return True
           
    ##-----------------------------------------------------------------------------------------------       
    def triangulate_mesh(self):
        bm = bmesh.new()
        bm.from_mesh(self.blender_mesh)
        bmesh.ops.triangulate(bm, faces = bm.faces[:])
        bm.to_mesh(self.blender_mesh)
        bm.free()
