from ..core.aabb import BoundingBox


#----------------------------------------------------------------------------------------------------
class SubmeshDescription():
    #------------------------------------------------------------------------------------------------
    def __init__(
        self
        ):
        self.channels                 = []
        self.index_count              = 0
        self.vertex_count             = 0
        self.aabb                     = BoundingBox(0,0)
        self.uv_distribution_metric   = 0 
        self.payload_size             = 0
       
    #------------------------------------------------------------------------------------------------
    def get_submesh_dict(self, submeshes, submesh_count):
        data = {}
        
        data["aabb"]        = {}
        data["aabb"]["max"] = submeshes[submesh_count].aabb.max
        data["aabb"]["min"] = submeshes[submesh_count].aabb.min
        data["channels"]    = []
        for channel in submeshes[submesh_count].channels:
            if channel != 0:
                channel_data = {
                    "dataOffset": channel.data_offset,
                    "dataSize": channel.data_size,
                    "elementCount": channel.element_count,
                    "elementSize": channel.element_size,
                    "elementType": channel.element_type,
                    "semantic": channel.semantic
                }
                data["channels"].append(channel_data)
                

        data["indexCount"]           = submeshes[submesh_count].index_count
        data["uvDistributionMetric"] = submeshes[submesh_count].uv_distribution_metric
        data["vertexCount"]          = submeshes[submesh_count].vertex_count
            
        return data   
    
##---------------------------------------------------------------------------------------------------
class MeshDescription():
    ##-----------------------------------------------------------------------------------------------
    def __init__(
        self, name, uuid, submesh_count):
        self.name                     = name
        self.uuid                     = uuid
        self.payload_size             = 0
        self.submeshes                = [SubmeshDescription() for _ in range(submesh_count)]
  
    ##-----------------------------------------------------------------------------------------------
    def get_description_dict(self):
        description = {}
        description['name']         = self.name
        description['payloadSize']  = self.payload_size
        
        submesh_dicts = []
        for i,item in enumerate(self.submeshes):
            submesh_dicts.append(item.get_submesh_dict(self.submeshes, i))
            
        description["submeshes"]    = submesh_dicts
        description['uuid']         = self.uuid
        description['version']      = 1
        
        return description
    